pub struct RankSelect<CC = Vec<u64>, VC = Vec<u64>, WC = [u64; 2]> {
pub counts: CC,
pub values: Bools<VC, WC>,
}Expand description
A store for maintaining Vec<bool> with fast rank and select access.
The design is to have u64 running counts for each block of 1024 bits,
which are roughly the size of a cache line. This is roughly 6% overhead,
above the bits themselves, which seems pretty solid.
Fields
counts: CCCounts of the number of cumulative set (true) bits, after each block of 1024 bits.
values: Bools<VC, WC>The bits themselves.
Implementations
Sourceimpl<CC: BorrowIndexAs<u64>, VC: BorrowIndexAs<u64>> RankSelect<CC, VC>
impl<CC: BorrowIndexAs<u64>, VC: BorrowIndexAs<u64>> RankSelect<CC, VC>
Sourceimpl<CC: Len + IndexAs<u64>, VC: Len + IndexAs<u64>, WC: IndexAs<u64>> RankSelect<CC, VC, WC>
impl<CC: Len + IndexAs<u64>, VC: Len + IndexAs<u64>, WC: IndexAs<u64>> RankSelect<CC, VC, WC>
Sourcepub fn rank(&self, index: usize) -> usize
pub fn rank(&self, index: usize) -> usize
The number of set bits strictly preceding index.
This number is accumulated first by reading out of self.counts at the correct position,
then by summing the ones in strictly prior u64 entries, then by counting the ones in the
masked u64 in which the bit lives.
Sourcepub fn select(&self, rank: u64) -> Option<usize>
pub fn select(&self, rank: u64) -> Option<usize>
The position of the rank-th set bit (0-indexed), if it exists.
select(0) returns the position of the first set bit. In general,
select(rank(p)) == p when p is the position of a set bit, mirroring
the convention of Self::rank (which counts bits strictly before
its argument).